// Copyright (C) 2006-2011 Xilinx, Inc.  All rights reserved
// $Id: PinGrid.h,v 1.11 2011/03/04 02:05:29 jcorbett Exp $
//
// R#check_package_pins, R#check_io_banks - Class used by PinChecker
// and ConstraintsChecker for package pin, IO buffer (die pin), and IO
// bank information.
#ifndef Iv_PinGrid_h
#define Iv_PinGrid_h

#include <map>
#include <string>

#include "TUtilities/export/smart_ptr.hpp"

#include "main.h"
#include "GridMap.h"
#include "PinGroup.h"

class DeviceModel;
class Rf_PackagePin;

class PinGrid {
public:
   PinGrid(const DeviceModel* deviceModel);
   ~PinGrid();
   void LabelPin(const Rf_PackagePin* pkgPin,
                 const std::string& net,
                 const std::string& group);
   void CheckPinLocations();
   void CheckIoBanks();
   void ReportFaults();
   int GetStatus();
   int GetErrorCount();
private:
   DeviceModel* deviceModel;
   std::vector<PinGroup*> ioBuffers;
   std::vector<PinGroup*> pkgPins;
   boost::shared_ptr<GridMap<PinGroup> > ioBufferGrid;
   boost::shared_ptr<GridMap<PinGroup> > pkgPinGrid;
   PinFaultList ioBufferFaults;
   PinFaultList pkgPinFaults;

   typedef std::string PinName;

   typedef std::map<std::string, PinGroup*> GroupPinMap;
   typedef GroupPinMap::iterator GroupPinMapIt;

   typedef std::map<int, GroupPinMap> BankGroup;
   typedef BankGroup::iterator BankGroupIt;
   BankGroup bankGroup;
   int bankFaultCount;
};

#endif
